@@ -218,7 +218,7 @@ EXTRA_CORS_ALLOWED_ORIGINS = os.environ.get("EXTRA_CORS_ALLOWED_ORIGINS", "").st
 DISABLE_EVENTS = is_env_true("DISABLE_EVENTS")
 DEBUG_ANALYTICS = is_env_true("DEBUG_ANALYTICS")
 
-# Whether to skip downloading additional infrastructure components (e.g., custom Elasticsearch versions)
+# whether to skip downloading additional infrastructure components (e.g., custom Elasticsearch versions)
 SKIP_INFRA_DOWNLOADS = os.environ.get("SKIP_INFRA_DOWNLOADS", "").strip()
 
 # Adding Stepfunctions default port
@@ -247,6 +247,9 @@ FORCE_SHUTDOWN = is_env_not_false("FORCE_SHUTDOWN")
 # whether the in_docker check should always return true
 OVERRIDE_IN_DOCKER = is_env_true("OVERRIDE_IN_DOCKER")
 
+# whether to return mocked success responses for still unimplemented API methods
+MOCK_UNIMPLEMENTED = is_env_true("MOCK_UNIMPLEMENTED")
+
 
 def has_docker():
     try:
