@@ -3,6 +3,7 @@ import os
 from localstack.services.awslambda.lambda_api import LAMBDA_POLICY_NAME_PATTERN
 from localstack.services.awslambda.lambda_utils import get_handler_file_from_name
 from localstack.services.cloudformation.deployment_utils import (
+    generate_default_name,
     get_cfn_response_mod_file,
     select_parameters,
 )
@@ -59,6 +60,14 @@ class LambdaFunction(GenericBaseModel):
             }
         return client.update_function_configuration(**update_props)
 
+    @staticmethod
+    def add_defaults(resource, stack_name: str):
+        role_name = resource.get("Properties", {}).get("FunctionName")
+        if not role_name:
+            resource["Properties"]["FunctionName"] = generate_default_name(
+                stack_name, resource["LogicalResourceId"]
+            )
+
     @staticmethod
     def get_deploy_templates():
         def get_lambda_code_param(params, **kwargs):
@@ -91,6 +100,13 @@ class LambdaFunction(GenericBaseModel):
         def get_delete_params(params, **kwargs):
             return {"FunctionName": params.get("FunctionName")}
 
+        def get_environment_params(params, **kwargs):
+            # botocore/data/lambda/2015-03-31/service-2.json:1161 (EnvironmentVariableValue)
+            # https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lambda-function-environment.html
+            if "Environment" in params:
+                environment_variables = params["Environment"].get("Variables", {})
+                return {"Variables": {k: str(v) for k, v in environment_variables.items()}}
+
         return {
             "create": {
                 "function": "create_function",
@@ -101,7 +117,7 @@ class LambdaFunction(GenericBaseModel):
                     "Handler": "Handler",
                     "Code": get_lambda_code_param,
                     "Description": "Description",
-                    "Environment": "Environment",
+                    "Environment": get_environment_params,
                     "Timeout": "Timeout",
                     "MemorySize": "MemorySize",
                     "Layers": "Layers"
