@@ -7,6 +7,7 @@ import signal
 import subprocess
 import sys
 import threading
+from typing import Dict, Optional
 
 from requests.models import Response
 
@@ -35,6 +36,7 @@ from localstack.utils.aws.aws_stack import (
     is_internal_call_context,
     set_default_region_in_headers,
 )
+from localstack.utils.aws.request_routing import extract_version_and_action, matches_service_action
 from localstack.utils.common import (
     TMP_THREADS,
     empty_context_manager,
@@ -98,13 +100,6 @@ class ProxyListenerEdge(ProxyListener):
         # extract API details
         api, port, path, host = get_api_from_headers(headers, method=method, path=path, data=data)
 
-        if api and is_trace_logging_enabled(headers):
-            # print request trace for debugging, if enabled
-            LOG.debug(
-                'IN(%s): "%s %s" - headers: %s - data: %s'
-                % (api, method, path, dict(headers), data)
-            )
-
         set_default_region_in_headers(headers)
 
         if port and int(port) < 0:
@@ -116,11 +111,19 @@ class ProxyListenerEdge(ProxyListener):
                 port,
             )
 
+        should_log_trace = is_trace_logging_enabled(headers)
+        if api and should_log_trace:
+            # print request trace for debugging, if enabled
+            LOG.debug(
+                'IN(%s): "%s %s" - headers: %s - data: %s'
+                % (api, method, path, dict(headers), data)
+            )
+
         if not port:
             if method == "OPTIONS":
-                if api and is_trace_logging_enabled(headers):
+                if api and should_log_trace:
                     # print request trace for debugging, if enabled
-                    LOG.debug('OUT(%s): "%s %s" - status: %s' % (api, method, path, 200))
+                    LOG.debug('IN(%s): "%s %s" - status: %s' % (api, method, path, 200))
                 return 200
 
             if api in ["", None, API_UNKNOWN]:
@@ -164,7 +167,21 @@ class ProxyListenerEdge(ProxyListener):
             lock_ctx = empty_context_manager()
 
         with lock_ctx:
-            return do_forward_request(api, method, path, data, headers, port=port)
+            result = do_forward_request(api, method, path, data, headers, port=port)
+            if should_log_trace and result not in [None, False, True]:
+                result_status_code = getattr(result, "status_code", result)
+                result_headers = getattr(result, "headers", {})
+                result_content = getattr(result, "content", "")
+                LOG.debug(
+                    'OUT(%s): "%s %s" - status: %s - response headers: %s - response: %s',
+                    api,
+                    method,
+                    path,
+                    result_status_code,
+                    dict(result_headers or {}),
+                    result_content,
+                )
+            return result
 
     def return_response(self, method, path, data, headers, response, request_handler=None):
         api = headers.get(HEADER_TARGET_API) or ""
@@ -297,8 +314,8 @@ def get_api_from_headers(headers, method=None, path=None, data=None):
 
     # https://docs.aws.amazon.com/general/latest/gr/sigv4-signed-request-examples.html
     try:
-        credential_scope = auth_header.split(",")[0].split()[1]
-        _, _, _, service, _ = credential_scope.split("/")
+        service = extract_service_name_from_auth_header(headers)
+        assert service
         result = service, get_service_port_for_account(service, headers)
     except Exception:
         pass
@@ -342,6 +359,16 @@ def get_api_from_headers(headers, method=None, path=None, data=None):
     return result[0], result_before[1] or result[1], path, host
 
 
+def extract_service_name_from_auth_header(headers: Dict) -> Optional[str]:
+    try:
+        auth_header = headers.get("authorization", "")
+        credential_scope = auth_header.split(",")[0].split()[1]
+        _, _, _, service, _ = credential_scope.split("/")
+        return service
+    except Exception:
+        return
+
+
 def is_s3_form_data(data_bytes):
     if to_bytes("key=") in data_bytes:
         return True
@@ -428,6 +455,10 @@ def get_api_from_custom_rules(method, path, data, headers):
         return "apigateway", config.PORT_APIGATEWAY
 
     data_bytes = to_bytes(data or "")
+    version, action = extract_version_and_action(path, data_bytes)
+
+    def _in_path_or_payload(search_str):
+        return to_str(search_str) in path or to_bytes(search_str) in data_bytes
 
     if path == "/" and b"QueueName=" in data_bytes:
         return "sqs", config.PORT_SQS
@@ -438,13 +469,10 @@ def get_api_from_custom_rules(method, path, data, headers):
     if path.startswith("/2015-03-31/functions/"):
         return "lambda", config.PORT_LAMBDA
 
-    if (
-        b"Action=AssumeRoleWithWebIdentity" in data_bytes
-        or "Action=AssumeRoleWithWebIdentity" in path
-    ):
+    if _in_path_or_payload("Action=AssumeRoleWithWebIdentity"):
         return "sts", config.PORT_STS
 
-    if b"Action=AssumeRoleWithSAML" in data_bytes or "Action=AssumeRoleWithSAML" in path:
+    if _in_path_or_payload("Action=AssumeRoleWithSAML"):
         return "sts", config.PORT_STS
 
     # CloudWatch backdoor API to retrieve raw metrics
@@ -452,10 +480,14 @@ def get_api_from_custom_rules(method, path, data, headers):
         return "cloudwatch", config.PORT_CLOUDWATCH
 
     # SQS queue requests
-    if ("QueueUrl=" in path and "Action=" in path) or (
-        b"QueueUrl=" in data_bytes and b"Action=" in data_bytes
-    ):
+    if _in_path_or_payload("QueueUrl=") and _in_path_or_payload("Action="):
         return "sqs", config.PORT_SQS
+    if matches_service_action("sqs", action, version=version):
+        return "sqs", config.PORT_SQS
+
+    # SNS topic requests
+    if matches_service_action("sns", action, version=version):
+        return "sns", config.PORT_SNS
 
     # TODO: move S3 public URLs to a separate port/endpoint, OR check ACLs here first
     stripped = path.strip("/")
@@ -580,8 +612,16 @@ def start_dns_server(asynchronous=False):
             if not asynchronous:
                 sleep_forever()
             return result
+
+        env_vars = {}
+        for env_var in config.CONFIG_ENV_VARS:
+            if env_var.startswith("DNS_"):
+                value = os.environ.get(env_var, None)
+                if value is not None:
+                    env_vars[env_var] = value
+
         # note: running in a separate process breaks integration with Route53 (to be fixed for local dev mode!)
-        return run_process_as_sudo("dns", PORT_DNS, asynchronous=asynchronous)
+        return run_process_as_sudo("dns", PORT_DNS, asynchronous=asynchronous, env_vars=env_vars)
     except Exception:
         pass
 
@@ -617,7 +657,7 @@ def start_edge(port=None, use_ssl=True, asynchronous=False):
     return run_process_as_sudo("edge", port, asynchronous=asynchronous)
 
 
-def run_process_as_sudo(component, port, asynchronous=False):
+def run_process_as_sudo(component, port, asynchronous=False, env_vars=None):
     # make sure we can run sudo commands
     try:
         ensure_can_use_sudo()
@@ -625,28 +665,38 @@ def run_process_as_sudo(component, port, asynchronous=False):
         LOG.error("cannot start service on privileged port %s: %s", port, str(e))
         return
 
+    # prepare environment
+    env_vars = env_vars or {}
+    env_vars["PYTHONPATH"] = f".:{LOCALSTACK_ROOT_FOLDER}"
+    env_vars["EDGE_FORWARD_URL"] = config.get_edge_url()
+    env_vars["EDGE_BIND_HOST"] = config.EDGE_BIND_HOST
+    env_vars_str = env_vars_to_string(env_vars)
+
     # start the process as sudo
-    sudo_cmd = "sudo -n "
+    sudo_cmd = "sudo -n"
     python_cmd = sys.executable
-    edge_url = config.get_edge_url()
-    cmd = "%sPYTHONPATH=.:%s EDGE_FORWARD_URL=%s EDGE_BIND_HOST=%s %s %s %s %s" % (
+    cmd = [
         sudo_cmd,
-        LOCALSTACK_ROOT_FOLDER,
-        edge_url,
-        config.EDGE_BIND_HOST,
+        env_vars_str,
         python_cmd,
         __file__,
         component,
-        port,
-    )
+        str(port),
+    ]
+    shell_cmd = " ".join(cmd)
 
     def run_command(*_):
-        run(cmd, outfile=subprocess.PIPE, print_error=False)
+        run(shell_cmd, outfile=subprocess.PIPE, print_error=False, env_vars=env_vars)
 
+    LOG.debug("Running command as sudo: %s", shell_cmd)
     result = start_thread(run_command, quiet=True) if asynchronous else run_command()
     return result
 
 
+def env_vars_to_string(env_vars: Dict) -> str:
+    return " ".join(f"{k}='{v}'" for k, v in env_vars.items())
+
+
 if __name__ == "__main__":
     logging.basicConfig()
     start_component(sys.argv[1], int(sys.argv[2]))
