@@ -77,12 +77,17 @@ URL_LOCALSTACK_FAT_JAR = (
 MARKER_FILE_LIGHT_VERSION = "%s/.light-version" % INSTALL_DIR_INFRA
 IMAGE_NAME_SFN_LOCAL = "amazon/aws-stepfunctions-local"
 ARTIFACTS_REPO = "https://github.com/localstack/localstack-artifacts"
-SFN_PATCH_CLASS = (
+SFN_PATCH_CLASS1 = "com/amazonaws/stepfunctions/local/runtime/Config.class"
+SFN_PATCH_CLASS2 = (
     "com/amazonaws/stepfunctions/local/runtime/executors/task/LambdaTaskStateExecutor.class"
 )
-SFN_PATCH_CLASS_URL = "%s/raw/master/stepfunctions-local-patch/%s" % (
+SFN_PATCH_CLASS_URL1 = "%s/raw/master/stepfunctions-local-patch/%s" % (
     ARTIFACTS_REPO,
-    SFN_PATCH_CLASS,
+    SFN_PATCH_CLASS1,
+)
+SFN_PATCH_CLASS_URL2 = "%s/raw/master/stepfunctions-local-patch/%s" % (
+    ARTIFACTS_REPO,
+    SFN_PATCH_CLASS2,
 )
 
 # kinesis-mock version
@@ -122,9 +127,6 @@ GLIBC_URL = "https://github.com/sgerrand/alpine-pkg-glibc/releases/download/%s/%
 GLIBC_PATH = config.TMP_FOLDER + "/" + GLIBC_FILE
 CA_CERTIFICATES = "ca-certificates"
 
-# set up logger
-LOG = logging.getLogger(__name__)
-
 
 def get_elasticsearch_install_version(version=None):
     if config.SKIP_INFRA_DOWNLOADS:
@@ -174,7 +176,7 @@ def install_elasticsearch(version=None):
             plugin_binary = os.path.join(install_dir, "bin", "elasticsearch-plugin")
             plugin_dir = os.path.join(install_dir, "plugins", plugin)
             if not os.path.exists(plugin_dir):
-                LOG.info("Installing Elasticsearch plugin %s" % (plugin))
+                LOG.info("Installing Elasticsearch plugin %s" % plugin)
 
                 def try_install():
                     safe_run([plugin_binary, "install", "-b", plugin])
@@ -334,15 +336,19 @@ def install_stepfunctions_local():
             file.rename(Path(INSTALL_DIR_STEPFUNCTIONS) / file.name)
         rm_rf("%s/stepfunctionslocal" % INSTALL_DIR_INFRA)
     # apply patches
-    patch_class_file = os.path.join(INSTALL_DIR_STEPFUNCTIONS, SFN_PATCH_CLASS)
-    if not os.path.exists(patch_class_file):
-        download(SFN_PATCH_CLASS_URL, patch_class_file)
-        cmd = 'cd "%s"; zip %s %s' % (
-            INSTALL_DIR_STEPFUNCTIONS,
-            INSTALL_PATH_STEPFUNCTIONS_JAR,
-            SFN_PATCH_CLASS,
-        )
-        run(cmd)
+    for patch_class, patch_url in (
+        (SFN_PATCH_CLASS1, SFN_PATCH_CLASS_URL1),
+        (SFN_PATCH_CLASS2, SFN_PATCH_CLASS_URL2),
+    ):
+        patch_class_file = os.path.join(INSTALL_DIR_STEPFUNCTIONS, patch_class)
+        if not os.path.exists(patch_class_file):
+            download(patch_url, patch_class_file)
+            cmd = 'cd "%s"; zip %s %s' % (
+                INSTALL_DIR_STEPFUNCTIONS,
+                INSTALL_PATH_STEPFUNCTIONS_JAR,
+                patch_class,
+            )
+            run(cmd)
 
 
 def install_dynamodb_local():
