@@ -1,18 +1,32 @@
-import os
-import types
 import logging
+import os
 import traceback
-from moto.sqs import responses as sqs_responses
-from moto.sqs.models import Queue
+import types
+from html import escape
+
 from moto.core.utils import camelcase_to_underscores
+from moto.sqs import responses as sqs_responses
 from moto.sqs.exceptions import QueueDoesNotExist
+from moto.sqs.models import Queue
+
 from localstack import config
 from localstack.config import LOCALSTACK_HOSTNAME, TMP_FOLDER
+from localstack.services.infra import (
+    do_run,
+    log_startup_message,
+    start_moto_server,
+    start_proxy_for_service,
+)
+from localstack.services.install import INSTALL_DIR_ELASTICMQ, SQS_BACKEND_IMPL, install_elasticmq
 from localstack.utils.aws import aws_stack
 from localstack.utils.common import (
-    wait_for_port_open, save_file, short_uid, TMP_FILES, get_free_tcp_port, to_str, escape_html)
-from localstack.services.infra import start_proxy_for_service, do_run, start_moto_server, log_startup_message
-from localstack.services.install import INSTALL_DIR_ELASTICMQ, SQS_BACKEND_IMPL, install_elasticmq
+    TMP_FILES,
+    get_free_tcp_port,
+    save_file,
+    short_uid,
+    to_str,
+    wait_for_port_open,
+)
 
 LOG = logging.getLogger(__name__)
 
@@ -20,7 +34,7 @@ LOG = logging.getLogger(__name__)
 PORT_SQS_BACKEND = None
 
 # max heap size allocated for the Java process
-MAX_HEAP_SIZE = '256m'
+MAX_HEAP_SIZE = "256m"
 
 
 def check_sqs(expect_shutdown=False, print_error=False):
@@ -30,18 +44,18 @@ def check_sqs(expect_shutdown=False, print_error=False):
         if PORT_SQS_BACKEND:
             wait_for_port_open(PORT_SQS_BACKEND)
         # check SQS
-        out = aws_stack.connect_to_service(service_name='sqs').list_queues()
+        out = aws_stack.connect_to_service(service_name="sqs").list_queues()
     except Exception as e:
         if print_error:
-            LOG.warning('SQS health check failed: %s %s' % (e, traceback.format_exc()))
+            LOG.warning("SQS health check failed: %s %s" % (e, traceback.format_exc()))
     if expect_shutdown:
         assert out is None
     else:
-        assert out.get('ResponseMetadata', {}).get('HTTPStatusCode') == 200
+        assert out.get("ResponseMetadata", {}).get("HTTPStatusCode") == 200
 
 
 def start_sqs(*args, **kwargs):
-    if SQS_BACKEND_IMPL == 'moto':
+    if SQS_BACKEND_IMPL == "moto":
         return start_sqs_moto(*args, **kwargs)
     return start_sqs_elasticmq(*args, **kwargs)
 
@@ -65,9 +79,7 @@ def patch_moto():
     def _set_attributes(self, attributes, now=None):
         _set_attributes_orig(self, attributes, now)
 
-        integer_fields = [
-            'ReceiveMessageWaitTimeSeconds'
-        ]
+        integer_fields = ["ReceiveMessageWaitTimeSeconds"]
 
         for key in integer_fields:
             attribute = camelcase_to_underscores(key)
@@ -81,14 +93,14 @@ def patch_moto():
 
         def _escape(val):
             try:
-                return val and escape_html(to_str(val))
+                return val and escape(to_str(val))
             except Exception:
                 return val
 
         def render(self, *args, **kwargs):
             return render_orig(*args, _escape=_escape, **kwargs)
 
-        if not hasattr(template, '__patched'):
+        if not hasattr(template, "__patched"):
             render_orig = template.render
             template.render = types.MethodType(render, template)
             template.__patched = True
@@ -99,20 +111,21 @@ def patch_moto():
 
     # escape message responses to allow for special characters like "<"
     sqs_responses.RECEIVE_MESSAGE_RESPONSE = sqs_responses.RECEIVE_MESSAGE_RESPONSE.replace(
-        '<StringValue>{{ value.string_value }}</StringValue>',
-        '<StringValue>{{ _escape(value.string_value) }}</StringValue>')
+        "<StringValue>{{ value.string_value }}</StringValue>",
+        "<StringValue>{{ _escape(value.string_value) }}</StringValue>",
+    )
 
     # Fix issue with trailing slash
     # https://github.com/localstack/localstack/issues/2874
     def sqs_responses_get_queue_name(self):
         try:
-            queue_url = self.querystring.get('QueueUrl')[0]
-            queue_name_data = queue_url.split('/')[4:]
+            queue_url = self.querystring.get("QueueUrl")[0]
+            queue_name_data = queue_url.split("/")[4:]
             queue_name_data = [queue_attr for queue_attr in queue_name_data if queue_attr]
-            queue_name = '/'.join(queue_name_data)
+            queue_name = "/".join(queue_name_data)
         except TypeError:
             # Fallback to reading from the URL
-            queue_name = self.path.split('/')[2]
+            queue_name = self.path.split("/")[2]
 
         if not queue_name:
             raise QueueDoesNotExist()
@@ -125,8 +138,13 @@ def patch_moto():
 def start_sqs_moto(port=None, asynchronous=False, update_listener=None):
     port = port or config.PORT_SQS
     patch_moto()
-    return start_moto_server('sqs', port, name='SQS',
-        asynchronous=asynchronous, update_listener=update_listener)
+    return start_moto_server(
+        "sqs",
+        port,
+        name="SQS",
+        asynchronous=asynchronous,
+        update_listener=update_listener,
+    )
 
 
 def start_sqs_elasticmq(port=None, asynchronous=False, update_listener=None):
@@ -150,13 +168,20 @@ def start_sqs_elasticmq(port=None, asynchronous=False, update_listener=None):
         bind-hostname = "0.0.0.0"
         sqs-limits = strict
     }
-    """ % (LOCALSTACK_HOSTNAME, port, PORT_SQS_BACKEND)
-    config_file = os.path.join(TMP_FOLDER, 'sqs.%s.conf' % short_uid())
+    """ % (
+        LOCALSTACK_HOSTNAME,
+        port,
+        PORT_SQS_BACKEND,
+    )
+    config_file = os.path.join(TMP_FOLDER, "sqs.%s.conf" % short_uid())
     TMP_FILES.append(config_file)
     save_file(config_file, config_params)
     # start process
-    cmd = ('java -Dconfig.file=%s -Xmx%s -jar %s/elasticmq-server.jar' % (
-        config_file, MAX_HEAP_SIZE, INSTALL_DIR_ELASTICMQ))
-    log_startup_message('SQS')
-    start_proxy_for_service('sqs', port, PORT_SQS_BACKEND, update_listener)
+    cmd = "java -Dconfig.file=%s -Xmx%s -jar %s/elasticmq-server.jar" % (
+        config_file,
+        MAX_HEAP_SIZE,
+        INSTALL_DIR_ELASTICMQ,
+    )
+    log_startup_message("SQS")
+    start_proxy_for_service("sqs", port, PORT_SQS_BACKEND, update_listener)
     return do_run(cmd, asynchronous)
