@@ -1,13 +1,23 @@
 import logging
 import re
 import threading
+from urllib.parse import urlparse
 
 from flask import request
+from moto.core import utils as moto_utils
 from requests.models import Request
 from requests.structures import CaseInsensitiveDict
 
 from localstack import config
-from localstack.utils.common import empty_context_manager
+from localstack.constants import APPLICATION_JSON, APPLICATION_XML, HEADER_CONTENT_TYPE
+from localstack.utils.aws import aws_stack
+from localstack.utils.aws.aws_responses import (
+    is_json_request,
+    requests_error_response,
+    requests_response,
+    requests_to_flask_response,
+)
+from localstack.utils.common import empty_context_manager, snake_to_camel_case
 from localstack.utils.run import FuncThread
 
 LOG = logging.getLogger(__name__)
@@ -108,7 +118,7 @@ def configure_region_for_current_request(region_name: str, service_name: str):
 
     headers = request_context.headers
     auth_header = headers.get("Authorization")
-    auth_header = auth_header or aws_stack.mock_aws_request_headers(service_name)
+    auth_header = auth_header or aws_stack.mock_aws_request_headers(service_name)["Authorization"]
     auth_header = auth_header.replace("/%s/" % aws_stack.get_region(), "/%s/" % region_name)
     try:
         headers["Authorization"] = auth_header
@@ -119,7 +129,51 @@ def configure_region_for_current_request(region_name: str, service_name: str):
         _context_to_update.headers = CaseInsensitiveDict({**headers, "Authorization": auth_header})
 
 
+def mock_request_for_region(region_name: str, service_name: str = "dummy") -> Request:
+    result = Request()
+    result.headers["Authorization"] = aws_stack.mock_aws_request_headers(
+        service_name, region_name=region_name
+    )["Authorization"]
+    return result
+
+
 def patch_request_handling():
+    from localstack.services.edge import (
+        extract_service_name_from_auth_header,  # leave here to avoid import issues
+    )
+
+    # make sure we properly handle/propagate "not implemented" errors
+    def convert_flask_to_httpretty_response_call(*args, **kwargs):
+        try:
+            return convert_flask_to_httpretty_response_call_orig(*args, **kwargs)
+        except NotImplementedError as e:
+            action = request.headers.get("X-Amz-Target")
+            action = action or f"{request.method} {urlparse(request.url).path}"
+            if action == "POST /":
+                # try to extract action from exception string
+                match = re.match(r"The ([a-zA-Z0-9_-]+) action has not been implemented", str(e))
+                if match:
+                    action = snake_to_camel_case(match.group(1))
+            service = extract_service_name_from_auth_header(request.headers)
+            msg = f"API action '{action}' for service '{service}' not yet implemented"
+            response = requests_error_response(request.headers, msg, code=501)
+            if config.MOCK_UNIMPLEMENTED:
+                is_json = is_json_request(request.headers)
+                headers = {HEADER_CONTENT_TYPE: APPLICATION_JSON if is_json else APPLICATION_XML}
+                content = "{}" if is_json else "<Response />"  # TODO: return proper mocked response
+                response = requests_response(content, headers=headers)
+                LOG.info(f"{msg}. Returning mocked response due to MOCK_UNIMPLEMENTED=1")
+            else:
+                LOG.info(msg)
+            # TODO: publish analytics event ...
+            return requests_to_flask_response(response)
+
+    convert_flask_to_httpretty_response_call_orig = (
+        moto_utils.convert_flask_to_httpretty_response.__call__
+    )
+    moto_utils.convert_flask_to_httpretty_response.__call__ = (
+        convert_flask_to_httpretty_response_call
+    )
 
     if config.USE_SINGLE_REGION:
         return
