@@ -3,7 +3,6 @@ import os
 import pkgutil
 import re
 import shlex
-import shutil
 import sys
 import threading
 import time
@@ -11,10 +10,10 @@ import warnings
 from datetime import datetime
 from functools import wraps
 
-import pip as pip_mod
 import six
 
 from localstack import config, constants
+from localstack.constants import LS_LOG_TRACE_INTERNAL, TRACE_LOG_LEVELS
 from localstack.utils.docker import DOCKER_CLIENT, ContainerException, PortMappings
 
 # set up logger
@@ -76,50 +75,6 @@ MAIN_CONTAINER_NAME_CACHED = None
 ENV_SCRIPT_STARTING_DOCKER = "LS_SCRIPT_STARTING_DOCKER"
 
 
-def bootstrap_installation():
-    try:
-        from localstack.services import infra
-
-        assert infra
-    except Exception:
-        install_dependencies()
-
-
-def install_dependencies():
-    # determine requirements
-    root_folder = os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "..")
-    reqs_file = os.path.join(root_folder, "requirements.txt")
-    reqs_copy_file = os.path.join(root_folder, "localstack", "requirements.copy.txt")
-    if not os.path.exists(reqs_copy_file):
-        shutil.copy(reqs_file, reqs_copy_file)
-    with open(reqs_copy_file) as f:
-        requirements = f.read()
-    install_requires = []
-    for line in re.split("\n", requirements):
-        if line and line[0] != "#":
-            if BASIC_LIB_MARKER not in line and IGNORED_LIB_MARKER not in line:
-                line = line.split(" #")[0].strip()
-                install_requires.append(line)
-    LOG.info(
-        "Lazily installing missing pip dependencies, this could take a while: %s"
-        % ", ".join(install_requires)
-    )
-    args = ["install"] + install_requires
-    return run_pip_main(args)
-
-
-def run_pip_main(args):
-    if hasattr(pip_mod, "main"):
-        return pip_mod.main(args)
-    import pip._internal
-
-    if hasattr(pip._internal, "main"):
-        return pip._internal.main(args)
-    import pip._internal.main
-
-    return pip._internal.main.main(args)
-
-
 def log_duration(name=None):
     """Function decorator to log the duration of function invocations."""
 
@@ -313,7 +268,7 @@ def setup_logging(log_level=None):
     # overriding the log level if LS_LOG has been set
     if config.LS_LOG:
         log_level = str(config.LS_LOG).upper()
-        if log_level == "TRACE":
+        if log_level.lower() in TRACE_LOG_LEVELS:
             log_level = "DEBUG"
         log_level = logging._nameToLevel[log_level]
         logging.getLogger("").setLevel(log_level)
@@ -342,12 +297,16 @@ def setup_logging(log_level=None):
     logging.captureWarnings(True)
     logging.getLogger("asyncio").setLevel(logging.INFO)
     logging.getLogger("boto3").setLevel(logging.INFO)
-    logging.getLogger("s3transfer").setLevel(logging.INFO)
-    logging.getLogger("docker").setLevel(logging.WARNING)
-    logging.getLogger("urllib3").setLevel(logging.WARNING)
-    logging.getLogger("requests").setLevel(logging.WARNING)
     logging.getLogger("botocore").setLevel(logging.ERROR)
+    logging.getLogger("docker").setLevel(logging.WARNING)
     logging.getLogger("elasticsearch").setLevel(logging.ERROR)
+    logging.getLogger("moto").setLevel(logging.WARNING)
+    logging.getLogger("requests").setLevel(logging.WARNING)
+    logging.getLogger("s3transfer").setLevel(logging.INFO)
+    logging.getLogger("urllib3").setLevel(logging.WARNING)
+    if config.LS_LOG != LS_LOG_TRACE_INTERNAL:
+        # disable werkzeug API logs, unless detailed internal trace logging is enabled
+        logging.getLogger("werkzeug").setLevel(logging.WARNING)
 
 
 # --------------
@@ -406,7 +365,6 @@ def is_api_enabled(api):
 
 
 def start_infra_locally():
-    bootstrap_installation()
     from localstack.services import infra
 
     return infra.start_infra()
@@ -547,6 +505,11 @@ def start_infra_in_docker():
 
     if DOCKER_CLIENT.is_container_running(container_name):
         raise Exception('LocalStack container named "%s" is already running' % container_name)
+    if config.TMP_FOLDER != config.HOST_TMP_FOLDER and not config.LAMBDA_REMOTE_DOCKER:
+        print(
+            f"WARNING: The detected temp folder for localstack ({config.TMP_FOLDER}) is not equal to the "
+            f"HOST_TMP_FOLDER environment variable set ({config.HOST_TMP_FOLDER})."
+        )  # Logger is not initialized at this point, so the warning is displayed via print
 
     os.environ[ENV_SCRIPT_STARTING_DOCKER] = "1"
 
