@@ -26,17 +26,12 @@ from localstack.services.cloudformation.service_models import (
     DependencyNotYetSatisfied,
     GenericBaseModel,
 )
-from localstack.services.s3 import s3_listener
 from localstack.utils import common
 from localstack.utils.aws import aws_stack
 from localstack.utils.cloudformation import template_preparer
 from localstack.utils.common import (
-    canonical_json,
     get_all_subclasses,
-    json_safe,
-    md5,
     prevent_stack_overflow,
-    short_uid,
     start_worker_thread,
     to_bytes,
     to_str,
@@ -587,11 +582,21 @@ def resolve_refs_recursively(stack_name, value, resources):
                 stack_name, option1 if condition else option2, resources
             )
 
+        if stripped_fn_lower == "condition":
+            result = evaluate_condition(stack_name, value[keys_list[0]], resources)
+            return result
+
         if stripped_fn_lower == "not":
             condition = value[keys_list[0]][0]
             condition = resolve_refs_recursively(stack_name, condition, resources)
             return not condition
 
+        if stripped_fn_lower in ["and", "or"]:
+            conditions = value[keys_list[0]]
+            results = [resolve_refs_recursively(stack_name, cond, resources) for cond in conditions]
+            result = all(results) if stripped_fn_lower == "and" else any(results)
+            return result
+
         if stripped_fn_lower == "equals":
             operand1, operand2 = value[keys_list[0]]
             operand1 = resolve_refs_recursively(stack_name, operand1, resources)
@@ -652,8 +657,9 @@ def resolve_placeholders_in_string(result, stack_name=None, resources=None):
             return resolved
         if len(parts) == 1 and parts[0] in resources:
             resource_json = resources[parts[0]]
+            resource_type = get_resource_type(resource_json)
             result = extract_resource_attribute(
-                resource_json.get("Type"),
+                resource_type,
                 {},
                 "Ref",
                 resources=resources,
@@ -718,7 +724,7 @@ def update_resource(resource_id, resources, stack_name):
         return result
 
 
-def get_resource_model_instance(resource_id, resources) -> GenericBaseModel:
+def get_resource_model_instance(resource_id: str, resources) -> Optional[GenericBaseModel]:
     """Obtain a typed resource entity instance representing the given stack resource."""
     resource = resources[resource_id]
     resource_type = get_resource_type(resource)
@@ -783,65 +789,6 @@ def deploy_resource(resource_id, resources, stack_name):
 
 
 def delete_resource(resource_id, resources, stack_name):
-    res = resources[resource_id]
-    res_type = res.get("Type")
-
-    if res_type == "AWS::S3::Bucket":
-        s3_listener.remove_bucket_notification(res["PhysicalResourceId"])
-
-    if res_type == "AWS::IAM::Role":
-        role_name = res.get("PhysicalResourceId") or res.get("Properties", {}).get("RoleName")
-        try:
-            iam_client = aws_stack.connect_to_service("iam")
-            rs = iam_client.list_role_policies(RoleName=role_name)
-            for policy in rs["PolicyNames"]:
-                iam_client.delete_role_policy(RoleName=role_name, PolicyName=policy)
-
-            rs = iam_client.list_instance_profiles_for_role(RoleName=role_name)
-            for instance_profile in rs["InstanceProfiles"]:
-                ip_name = instance_profile["InstanceProfileName"]
-                iam_client.remove_role_from_instance_profile(
-                    InstanceProfileName=ip_name, RoleName=role_name
-                )
-                # iam_client.delete_instance_profile(
-                #     InstanceProfileName=ip_name
-                # )
-
-        except Exception as e:
-            if "NoSuchEntity" not in str(e):
-                raise
-
-    if res_type == "AWS::EC2::VPC":
-        state = res[KEY_RESOURCE_STATE]
-        physical_resource_id = res["PhysicalResourceId"] or state.get("VpcId")
-        res["PhysicalResourceId"] = physical_resource_id
-
-        if state.get("VpcId"):
-            ec2_client = aws_stack.connect_to_service("ec2")
-            resp = ec2_client.describe_route_tables(
-                Filters=[
-                    {"Name": "vpc-id", "Values": [state.get("VpcId")]},
-                    {"Name": "association.main", "Values": ["false"]},
-                ]
-            )
-            for rt in resp["RouteTables"]:
-                ec2_client.delete_route_table(RouteTableId=rt["RouteTableId"])
-
-    if res_type == "AWS::EC2::Subnet":
-        state = res[KEY_RESOURCE_STATE]
-        physical_resource_id = res["PhysicalResourceId"] or state["SubnetId"]
-        res["PhysicalResourceId"] = physical_resource_id
-
-    if res_type == "AWS::EC2::RouteTable":
-        ec2_client = aws_stack.connect_to_service("ec2")
-        resp = ec2_client.describe_vpcs()
-        vpcs = [vpc["VpcId"] for vpc in resp["Vpcs"]]
-
-        vpc_id = res.get("Properties", {}).get("VpcId")
-        if vpc_id not in vpcs:
-            # VPC already deleted before
-            return
-
     return execute_resource_action(resource_id, resources, stack_name, ACTION_DELETE)
 
 
@@ -900,31 +847,6 @@ def execute_resource_action(resource_id, resources, stack_name, action_name):
     return (results or [None])[0]
 
 
-def fix_resource_props_for_sdk_deployment(resource_type, resource_props):
-    if resource_type == "Lambda::Function":
-        # Properties will be validated by botocore before sending request to AWS
-        # botocore/data/lambda/2015-03-31/service-2.json:1161 (EnvironmentVariableValue)
-        # https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lambda-function-environment.html
-        if "Environment" in resource_props:
-            environment_variables = resource_props["Environment"].get("Variables", {})
-            resource_props["Environment"]["Variables"] = {
-                k: str(v) for k, v in environment_variables.items()
-            }
-
-    if resource_type == "SQS::Queue":
-        # https://github.com/localstack/localstack/issues/3004
-        if "ReceiveMessageWaitTimeSeconds" in resource_props:
-            resource_props["ReceiveMessageWaitTimeSeconds"] = int(
-                resource_props["ReceiveMessageWaitTimeSeconds"]
-            )
-
-    if resource_type == "KMS::Key":
-        resource_props["KeyPolicy"] = json.dumps(resource_props.get("KeyPolicy", {}))
-        resource_props["Enabled"] = resource_props.get("Enabled", True)
-        resource_props["EnableKeyRotation"] = resource_props.get("EnableKeyRotation", False)
-        resource_props["Description"] = resource_props.get("Description", "")
-
-
 def configure_resource_via_sdk(
     resource_id, resources, resource_type, func_details, stack_name, action_name
 ):
@@ -942,9 +864,6 @@ def configure_resource_via_sdk(
     resource_props = dict(resource_props)
     resource_state = resource.get(KEY_RESOURCE_STATE, {})
 
-    # Validate props for each resource type
-    fix_resource_props_for_sdk_deployment(resource_type, resource_props)
-
     if callable(params):
         params = params(
             resource_props,
@@ -1068,7 +987,7 @@ def determine_resource_physical_id(
     resource = resources.get(resource_id, {})
     if not resource:
         return
-    resource_type = resource.get("Type") or ""
+    resource_type = get_resource_type(resource)
     resource_type = re.sub("^AWS::", "", resource_type)
     resource_props = resource.get("Properties", {})
 
@@ -1215,117 +1134,10 @@ def add_default_resource_props(
     """Apply some fixes to resource props which otherwise cause deployments to fail"""
 
     res_type = resource["Type"]
-    props = resource["Properties"] = resource.get("Properties", {})
-    existing_resources = existing_resources or {}
-
-    def _generate_res_name():
-        return "%s-%s-%s" % (stack_name, resource_name or resource_id, short_uid())
-
-    # TODO: move logic below into resource classes!
-
-    if res_type == "AWS::Lambda::EventSourceMapping" and not props.get("StartingPosition"):
-        props["StartingPosition"] = "LATEST"
-
-    elif res_type == "AWS::Logs::LogGroup" and not props.get("LogGroupName") and resource_name:
-        props["LogGroupName"] = resource_name
-
-    elif res_type == "AWS::Lambda::Function" and not props.get("FunctionName"):
-        # FunctionName is up to 64 characters long
-        random_id_part = short_uid()
-        resource_id_part = resource_id[:24]
-        stack_name_part = stack_name[: 63 - 2 - (len(random_id_part) + len(resource_id_part))]
-        props["FunctionName"] = f"{stack_name_part}-{resource_id_part}-{random_id_part}"
-
-    elif res_type == "AWS::SNS::Topic" and not props.get("TopicName"):
-        props["TopicName"] = "topic-%s" % short_uid()
-
-    elif res_type == "AWS::SQS::Queue" and not props.get("QueueName"):
-        props["QueueName"] = "queue-%s" % short_uid()
-
-    elif res_type == "AWS::IAM::ManagedPolicy" and not resource.get("ManagedPolicyName"):
-        resource["ManagedPolicyName"] = _generate_res_name()
-
-    elif res_type == "AWS::ApiGateway::RestApi" and not props.get("Name"):
-        props["Name"] = _generate_res_name()
-
-    elif res_type == "AWS::ApiGateway::Stage" and not props.get("StageName"):
-        props["StageName"] = "default"
-
-    elif res_type == "AWS::ApiGateway::ApiKey" and not props.get("Name"):
-        props["Name"] = _generate_res_name()
-
-    elif res_type == "AWS::ApiGateway::UsagePlan" and not props.get("UsagePlanName"):
-        props["UsagePlanName"] = _generate_res_name()
-
-    elif res_type == "AWS::ApiGateway::Model" and not props.get("Name"):
-        props["Name"] = _generate_res_name()
-
-    elif res_type == "AWS::ApiGateway::RequestValidator" and not props.get("Name"):
-        props["Name"] = _generate_res_name()
-
-    elif res_type == "AWS::DynamoDB::Table":
-        update_dynamodb_index_resource(resource)
-        props["TableName"] = props.get("TableName") or _generate_res_name()
-
-    elif res_type == "AWS::CloudWatch::Alarm":
-        props["AlarmName"] = props.get("AlarmName") or _generate_res_name()
-
-    elif res_type == "AWS::SecretsManager::Secret":
-        props["Name"] = props.get("Name") or _generate_res_name()
-
-    elif res_type == "AWS::S3::Bucket" and not props.get("BucketName"):
-        existing_bucket = existing_resources.get(resource_id) or {}
-        bucket_name = (
-            existing_bucket.get("Properties", {}).get("BucketName") or _generate_res_name()
-        )
-        props["BucketName"] = s3_listener.normalize_bucket_name(bucket_name)
-
-    elif res_type == "AWS::StepFunctions::StateMachine" and not props.get("StateMachineName"):
-        props["StateMachineName"] = _generate_res_name()
-
-    elif res_type == "AWS::CloudFormation::Stack" and not props.get("StackName"):
-        props["StackName"] = _generate_res_name()
-
-    elif res_type == "AWS::EC2::SecurityGroup":
-        props["GroupName"] = props.get("GroupName") or _generate_res_name()
-
-    elif res_type == "AWS::Redshift::Cluster":
-        props["ClusterIdentifier"] = props.get("ClusterIdentifier") or _generate_res_name()
-
-    elif res_type == "AWS::IAM::InstanceProfile":
-        props["InstanceProfileName"] = props.get("InstanceProfileName") or _generate_res_name()
-
-    elif res_type == "AWS::Logs::LogGroup":
-        props["LogGroupName"] = props.get("LogGroupName") or _generate_res_name()
-
-    elif res_type == "AWS::KMS::Key":
-        tags = props["Tags"] = props.get("Tags", [])
-        existing = [t for t in tags if t["Key"] == "localstack-key-id"]
-        if not existing:
-            # append tags, to allow us to determine in service_models.py whether this key is already deployed
-            tags.append({"Key": "localstack-key-id", "Value": short_uid()})
-
-    elif res_type == "AWS::IAM::ManagedPolicy":
-        props["ManagedPolicyName"] = props.get("ManagedPolicyName") or _generate_res_name()
-
-    # generate default names for certain resource types
-    default_attrs = (("AWS::IAM::Role", "RoleName"), ("AWS::Events::Rule", "Name"))
-    for entry in default_attrs:
-        if res_type == entry[0] and not props.get(entry[1]):
-            if not resource_id:
-                resource_id = canonical_json(json_safe(props))
-                resource_id = md5(resource_id)
-            props[entry[1]] = "cf-%s-%s" % (stack_name, resource_id)
-
-
-def update_dynamodb_index_resource(resource):
-    if resource.get("Properties").get("BillingMode") == "PAY_PER_REQUEST":
-        for glob_index in resource.get("Properties", {}).get("GlobalSecondaryIndexes", []):
-            if not glob_index.get("ProvisionedThroughput"):
-                glob_index["ProvisionedThroughput"] = {
-                    "ReadCapacityUnits": 99,
-                    "WriteCapacityUnits": 99,
-                }
+    canonical_type = canonical_resource_type(res_type)
+    resource_class = RESOURCE_MODELS.get(canonical_type)
+    if resource_class is not None:
+        resource_class.add_defaults(resource, stack_name)
 
 
 # -----------------------
@@ -1702,7 +1514,7 @@ class TemplateDeployer(object):
         # run deployment in background loop, to avoid client network timeouts
         return start_worker_thread(_run)
 
-    def do_apply_changes_in_loop(self, changes, stack, stack_name):
+    def do_apply_changes_in_loop(self, changes, stack, stack_name: str):
         # apply changes in a retry loop, to resolve resource dependencies and converge to the target state
         changes_done = []
         max_iters = 30
@@ -1734,7 +1546,7 @@ class TemplateDeployer(object):
                             resource_id, change, stack, new_resources
                         )
                         LOG.debug(
-                            'Handling "%s" for resource "%s" (%s/%s) type "%s" in loop iteration %s'
+                            'Handling "%s" for resource "%s" (%s/%s) type "%s" in loop iteration %s (should_deploy=%s)'
                             % (
                                 action,
                                 resource_id,
@@ -1742,6 +1554,7 @@ class TemplateDeployer(object):
                                 len(changes),
                                 res_change["ResourceType"],
                                 i + 1,
+                                should_deploy,
                             )
                         )
                         if not should_deploy:
