@@ -26,13 +26,25 @@ def run(
     inherit_env=True,
     tty=False,
     shell=True,
-):
+) -> Union[str, subprocess.Popen]:
     LOG.debug("Executing command: %s", cmd)
     env_dict = os.environ.copy() if inherit_env else {}
     if env_vars:
         env_dict.update(env_vars)
     env_dict = dict([(k, to_str(str(v))) for k, v in env_dict.items()])
 
+    if isinstance(cmd, list):
+        # See docs of subprocess.Popen(...):
+        #  "On POSIX with shell=True, the shell defaults to /bin/sh. If args is a string,
+        #   the string specifies the command to execute through the shell. [...] If args is
+        #   a sequence, the first item specifies the command string, and any additional
+        #   items will be treated as additional arguments to the shell itself."
+        # Hence, we should *disable* shell mode here to be on the safe side, to prevent
+        #  arguments in the cmd list from leaking into arguments to the shell itself. This will
+        #  effectively allow us to call run(..) with both - str and list - as cmd argument, although
+        #  over time we should move from "cmd: Union[str, List[str]]" to "cmd: List[str]" only.
+        shell = False
+
     if tty:
         asynchronous = True
         stdin = True
@@ -97,22 +109,22 @@ def run(
         raise e
 
 
-def is_mac_os():
+def is_mac_os() -> bool:
     return "Darwin" in get_uname()
 
 
-def is_linux():
+def is_linux() -> bool:
     return "Linux" in get_uname()
 
 
-def get_uname():
+def get_uname() -> str:
     try:
         return to_str(subprocess.check_output("uname -a", shell=True))
     except Exception:
         return ""
 
 
-def to_str(obj, errors="strict"):
+def to_str(obj: Union[str, bytes], errors="strict"):
     return obj.decode(config.DEFAULT_ENCODING, errors) if isinstance(obj, bytes) else obj
 
 
